@@ -1,3 +1,5 @@
+pub(crate) mod fs;
 pub(crate) mod random;
 pub(crate) mod signal_channel;
+pub(crate) mod sync;
 pub(crate) mod thread_handler;
